import os


BASE_DIR = os.path.dirname(os.path.abspath(__file__))
BLOCKS_DIR = os.path.join(BASE_DIR, 'blocks')
SIGNED_REQUESTS_DIR = os.path.join(BASE_DIR, 'signed_requests')
SIGNING_KEY_DIR = os.path.join(BASE_DIR, 'signing_keys')

# Account numbers
BANK_ACCOUNT_NUMBER = '5e12967707909e62b2bb2036c209085a784fabbc3deccefee70052b6181c8ed8'
BANK_NID_ACCOUNT_NUMBER = 'd5356888dc9303e44ce52b1e06c3165a7759b9df1e6a6dfbd33ee1c3df1ab4d1'
CV_ACCOUNT_NUMBER = '4d2ec91f37bc553bc538e91195669b666e26b2ea3e4e31507e38102a758d4f86'
CV_NID_ACCOUNT_NUMBER = '59479a31c3b91d96bb7a0b3e07f18d4bf301f1bb0bde05f8d36d9611dcbe7cbf'
BUCKY_ACCOUNT_NUMBER = '484b3176c63d5f37d808404af1a12c4b9649cd6f6769f35bdf5a816133623fbc'
CHRISTOPHER_ACCOUNT_NUMBER = 'a29baa6ba36f6db707f8f8dacfa82d5e8a28fa616e8cc96cf6d7790f551d79f2'
JUSTIN_ACCOUNT_NUMBER = '3214108063cda7b259782c57ff8cec343ad2f1ad35baf38c3503db5cf6f3b2f7'
KRISTY_ACCOUNT_NUMBER = 'db1a9ac3c356ab744ab4ad5256bb86c2f6dfaa7c1aece1f026a08dbd8c7178f2'
TREASURY_ACCOUNT_NUMBER = '0cdd4ba04456ca169baca3d66eace869520c62fe84421329086e03d91a68acdb'
PV_ACCOUNT_NUMBER = 'ad1f8845c6a1abb6011a2a434a079a087c460657aad54329a84b406dce8bf314'
PV_NID_ACCOUNT_NUMBER = '3afdf37573f1a511def0bd85553404b7091a76bcd79cdcebba1310527b167521'

# Bank fees
BANK_TX_FEE = 1

# Validator fees
PV_TX_FEE = 4



from rest_framework import serializers

from thenewboston.constants.network import MAX_POINT_VALUE, MIN_POINT_VALUE, VERIFY_KEY_LENGTH


class NetworkTransactionSerializer(serializers.Serializer):
    amount = serializers.IntegerField(max_value=MAX_POINT_VALUE, min_value=MIN_POINT_VALUE)
    recipient = serializers.CharField(max_length=VERIFY_KEY_LENGTH)

    def create(self, validated_data):
        pass

    def update(self, instance, validated_data):
        pass

    @staticmethod
    def validate_amount(amount):
        """
        Check that amount is not 0
        """

        if amount == 0:
            raise serializers.ValidationError('Tx amount can not be 0 (Tx should be excluded)')

        return amount

    @staticmethod
    def validate_recipient(recipient):
        """
        Check recipient length
        """

        if len(recipient) != VERIFY_KEY_LENGTH:
            raise serializers.ValidationError(f'recipient must be {VERIFY_KEY_LENGTH} characters long')

        return recipient



import os

from playground.config import SIGNING_KEY_DIR
from thenewboston.accounts.key_files import create_account_and_save_signing_key_file, read_signing_key_file
from thenewboston.verify_keys.verify_key import encode_verify_key, get_verify_key


def run():
    """
    Create new account and save signing key
    """

    file_path = os.path.join(SIGNING_KEY_DIR, 'cv_nid')
    create_account_and_save_signing_key_file(file=file_path)

    signing_key = read_signing_key_file(file_path)
    verify_key = get_verify_key(signing_key=signing_key)
    verify_key = encode_verify_key(verify_key=verify_key)

    print(f'verify_key: {verify_key}')


if __name__ == '__main__':
    run()



import os

from playground.config import (
    BANK_ACCOUNT_NUMBER,
    BANK_TX_FEE,
    BLOCKS_DIR,
    BUCKY_ACCOUNT_NUMBER,
    PV_ACCOUNT_NUMBER,
    PV_TX_FEE,
    SIGNING_KEY_DIR
)
from thenewboston.accounts.key_files import read_signing_key_file
from thenewboston.blocks.block import generate_block
from thenewboston.utils.files import write_json
from thenewboston.verify_keys.verify_key import get_verify_key


def run():
    """
    Create block used for:
    - POST /blocks
    """

    signing_key = read_signing_key_file(os.path.join(SIGNING_KEY_DIR, 'bucky'))
    account_number = get_verify_key(signing_key=signing_key)
    transactions = [
        {
            'amount': BANK_TX_FEE,
            'recipient': BANK_ACCOUNT_NUMBER,
        },
        {
            'amount': PV_TX_FEE,
            'recipient': PV_ACCOUNT_NUMBER,
        },
        {
            'amount': 2,
            'recipient': BUCKY_ACCOUNT_NUMBER,
        }
    ]
    block = generate_block(
        account_number=account_number,
        balance_lock='8db1bd4404eadce824b8f7af2461976e1fde8ebd7a279a2ccec2614479ede338',
        signing_key=signing_key,
        transactions=transactions
    )

    print(block)

    write_json(
        os.path.join(BLOCKS_DIR, 'upw.json'),
        block
    )


if __name__ == '__main__':
    run()



import os

from playground.config import (
    BANK_ACCOUNT_NUMBER,
    BANK_TX_FEE,
    BLOCKS_DIR,
    BUCKY_ACCOUNT_NUMBER,
    PV_ACCOUNT_NUMBER,
    PV_TX_FEE,
    SIGNING_KEY_DIR,
    TREASURY_ACCOUNT_NUMBER
)
from playground.utils import get_account_balance_lock
from thenewboston.accounts.key_files import read_signing_key_file
from thenewboston.blocks.block import generate_block
from thenewboston.utils.files import write_json
from thenewboston.utils.format import format_address
from thenewboston.utils.messages import get_message_hash
from thenewboston.utils.network import post
from thenewboston.verify_keys.verify_key import get_verify_key


def run(send_to_bank=False):
    """
    Create block used for:
    - POST /blocks
    """

    signing_key = read_signing_key_file(os.path.join(SIGNING_KEY_DIR, 'treasury'))
    account_number = get_verify_key(signing_key=signing_key)

    balance_lock = get_account_balance_lock(account_number=TREASURY_ACCOUNT_NUMBER, live_pv=True)
    print(f'\nCurrent balance lock: {balance_lock}')

    transactions = [
        {
            'amount': BANK_TX_FEE,
            'recipient': BANK_ACCOUNT_NUMBER,
        },
        {
            'amount': PV_TX_FEE,
            'recipient': PV_ACCOUNT_NUMBER,
        },
        {
            'amount': 5,
            'recipient': BUCKY_ACCOUNT_NUMBER,
        }
    ]
    block = generate_block(
        account_number=account_number,
        balance_lock=balance_lock,
        signing_key=signing_key,
        transactions=transactions
    )

    write_json(
        os.path.join(BLOCKS_DIR, 'blocks.json'),
        block
    )

    if send_to_bank:
        send_block_to_bank(block)


def send_block_to_bank(block):
    """
    Send block to bank
    """

    next_balance_lock = get_message_hash(message=block['message'])
    print(f'Next balance lock: {next_balance_lock}\n')

    bank_address = format_address(
        ip_address='167.99.173.247',
        port=None,
        protocol='http'
    )
    url = f'{bank_address}/blocks'
    results = post(url=url, body=block)

    if isinstance(results, dict):
        for k, v in results.items():
            print(f'{k}: {v}')

    write_json(
        os.path.join(BLOCKS_DIR, 'blocks-response.json'),
        results
    )


if __name__ == '__main__':
    run(send_to_bank=True)



from rest_framework import serializers

from thenewboston.constants.network import SIGNATURE_LENGTH, VERIFY_KEY_LENGTH
from thenewboston.serializers.message import MessageSerializer


class NetworkBlockSerializer(serializers.Serializer):
    account_number = serializers.CharField(max_length=VERIFY_KEY_LENGTH)
    message = MessageSerializer()
    signature = serializers.CharField(max_length=SIGNATURE_LENGTH)

    def create(self, validated_data):
        pass

    def update(self, instance, validated_data):
        pass



from setuptools import setup, find_packages

setup(
    install_requires=[
        'Django==3.1',
        'PTable==0.9.2',
        'PyNaCl==1.3.0',
        'django-cors-headers==3.4.0',
        'djangorestframework==3.11.1',
        'factory-boy==3.0.1',
        'pycodestyle==2.6.0',
        'pytest-django==3.9.0',
        'pytest==6.0.1',
        'requests~=2.23.0',
    ],
    name='thenewboston',
    packages=find_packages(
        exclude=['tests', 'tests.*']
    ),
    version='0.0.12',
)



import os

from playground.config import SIGNING_KEY_DIR
from thenewboston.accounts.key_files import read_signing_key_file
from thenewboston.blocks.signatures import generate_signature
from thenewboston.utils.tools import sort_and_encode


def run():
    """
    Run main script
    """

    payload = {
        "balance_key": "6cb8f4fe23c57a1c169e5a193c59ad9f21bbe5a54d69c44a4522f4644bf5b2a2",
        "txs": [
            {
                "amount": 5,
                "recipient": "484b3176c63d5f37d808404af1a12c4b9649cd6f6769f35bdf5a816133623fbc"
            },
            {
                "amount": 1,
                "recipient": "5e12967707909e62b2bb2036c209085a784fabbc3deccefee70052b6181c8ed8"
            },
            {
                "amount": 4,
                "recipient": "ad1f8845c6a1abb6011a2a434a079a087c460657aad54329a84b406dce8bf314"
            }
        ]
    }
    print(sort_and_encode(payload))
    signing_key = read_signing_key_file(os.path.join(SIGNING_KEY_DIR, 'sample'))
    signature = generate_signature(
        message=sort_and_encode(payload),
        signing_key=signing_key
    )
    print(signature)


if __name__ == '__main__':
    run()



from playground.config import TREASURY_ACCOUNT_NUMBER
from thenewboston.utils.format import format_address
from thenewboston.utils.network import fetch


def get_account_balance_lock(*, account_number, live_pv=False):
    """
    Return the balance lock for the given account
    """

    if live_pv:
        pv_address = format_address(
            ip_address='64.225.47.205',
            port=None,
            protocol='http'
        )
    else:
        pv_address = format_address(
            ip_address='192.168.1.75',
            port=8000,
            protocol='http'
        )

    url = f'{pv_address}/account_balance_lock/{account_number}'
    results = fetch(url=url, headers={})
    return results['balance_lock']


if __name__ == '__main__':
    get_account_balance_lock(account_number=TREASURY_ACCOUNT_NUMBER)



import os

from playground.config import BANK_NID_ACCOUNT_NUMBER, SIGNED_REQUESTS_DIR, SIGNING_KEY_DIR
from thenewboston.accounts.key_files import read_signing_key_file
from thenewboston.utils.files import write_json
from thenewboston.utils.format import format_address
from thenewboston.utils.network import patch
from thenewboston.utils.signed_requests import generate_signed_request

"""
Update trust for a bank
PATCH /banks/<node_identifier>
"""


def run(send_to_node=False):
    """
    Generate signed PATCH request
    """

    # Signed request
    sk = read_signing_key_file(os.path.join(SIGNING_KEY_DIR, 'bank_nid'))

    payload = {
        'trust': 76.26
    }

    target_nid = BANK_NID_ACCOUNT_NUMBER

    signed_request = generate_signed_request(
        data=payload,
        nid_signing_key=sk
    )

    if send_to_node:
        send_request_to_node(signed_request, target_nid)

    write_json(
        os.path.join(SIGNED_REQUESTS_DIR, 'bank-patch-request.json'),
        signed_request
    )


def send_request_to_node(signed_request, target_nid):
    """
    Send request to node
    """

    node_address = format_address(
        ip_address='192.168.1.232',
        port=8000,
        protocol='http'
    )
    url = f'{node_address}/banks/{target_nid}'
    results = patch(url=url, body=signed_request)

    if isinstance(results, dict):
        for k, v in results.items():
            print(f'{k}: {v}')

    print(results)

    write_json(
        os.path.join(SIGNED_REQUESTS_DIR, 'bank-patch-response.json'),
        results
    )


if __name__ == '__main__':
    run(send_to_node=True)
